c      Program FindMatch
c
c      Purpose: To find the station which is the "best match" to a set
c               of specified parameter values among the Cligen station
c               parameter files.
c
c      Author: Charles R. Meyer
c
c      Date: 8/25/2003 - 8/26/2003; 09/10/2003 - 9/15/2003; 06/30/2005
c
c      Version: 1.003
c      Changes:
c          - Added error messages to make it more clear what might be wrong
c            when a .TOP file cannot be processed.
c
c      Version: 1.002
c      Changes:
c          - Adapted FindMatch to match Southern Hemisphere stations to U. S.
c            (Northern Hemisphere) stations.  Requires a 6-month offset.
c          - Changed the matching algorithm to be a _normalized_ sum of squares.
c            CRM -- 10/30/03
c
c      Version: 1.001
c      Changes:
c          - Added the string "TYPE= 0" to GenStPar, and replaced the '0'
c            with the value read from the surrogate station in FindMatch.
c          - Expanded station names shown on the screen, from 24 characters
c            to the full 46, to accomodate the WEPP graphic user interface.
c            CRM -- 9/24/03
c
      include 'cbenchmk.inc'
      include 'cmatstat.inc'
c      stanam -- Station Name
c        latt -- Station Latitude
c      monpcp -- Monthly average precip per event
c        p_ww -- Monthly probability of wet after wet
c        p_wd -- Monthly probability of wet after dry
c          pw -- Monthly probability of precip
c       t_max -- Monthly average maximum temperature
c       t_min -- Monthly average minimum temperature
c      pcpamt -- Average amount of precip per month
c      annpcp -- Average amount of precip per year
c      anntmx -- Average maximum temperature per year
c      anntmn -- Average minimum temperature per year
c     keepers -- Names of stations showing best match
c     keepmet -- Metrics associated with each "keeper"
c
      real matmet
      character*6 latts, respon
      character*20 outfile
      character*46 snsave
      real kmsave,pcpsav(12),tmxsav(12),tmnsav(12),pwdsav(12),pwwsav(12)
      real lattsav, elevsav
c
c
c      matmet -- Metric used to determine quality of match
c       latts -- Character string to match to " LATT="
c
c
c  TEMPORARY.  REMOVE LATER:
C     infile = 'MS226009.PAR'
Cc    infile = 'CHONGQING.PAR'
CC    xinfile = '06235.TOP'
      write(*,*) 
      write(*,*) 'Welcome to the FindMatch program version 1.003'
      write(*,*) 'produced at the National Soil Erosion Research Lab'
      write(*,*) 'USDA-ARS, West Lafayette, Indiana, USA -- June 2005'
      write(*,*) 
      write(*,*) 'This program locates complete US climate station'
      write(*,*) 'files to match the partial Cligen station datafiles'
      write(*,*) 'built from the daily temperature and precipitation'
      write(*,*) 'data by the program GenStPar.  (These materials are'
      write(*,*) 'available from the Cligen website'
      write(*,*) 'http:horizon.nserl.purdue.edu/Cligen/).'
      write(*,*) 
      write(*,*) 'Please enter the name of your partial datafile.'
      write(*,*) '(Example: 06235.TOP)'
      write(*,*) 
      read(*,'(a)') xinfile
      write(*,*)
      open(unit=10, file=xinfile, status='old', err=9095)
c
c ---- Reading from .top file:
      read(10,2000,err=9987) xstanam
      read(10,2001,err=9988) latt
      read(10,2002,err=9989) elev
      read(10,2003,err=9990) monpcp
      read(10,2004,err=9991) 
      read(10,2005,err=9992) 
      read(10,2006,err=9993) p_ww
      read(10,2007,err=9994) p_wd
      read(10,2008,err=9995) t_max
      read(10,2009,err=9996) t_min
      read(10,2010,err=9997) sd_tmx
      read(10,2011,err=9998) sd_tmn
c ---- Reading from .top or .par file:
 2000 format(1x,a46)
 2001 format(' LATT=',f7.2,' LONG=',f7.2,' YEARS=',f4.0,' TYPE=',i2)
 2002 format(' ELEVATION =',f6.0)
 2003 format(' MEAN P ',12f6.2)
 2004 format(' S DEV P',12f6.2)
 2005 format(' SKEW P ',12f6.2)
 2006 format(' P(W|W) ',12f6.2)
 2007 format(' P(W|D) ',12f6.2)
 2008 format(' TMAX AV',12f6.2)
 2009 format(' TMIN AV',12f6.2)
 2010 format(' SD TMAX',12f6.2)
 2011 format(' SD TMIN',12f6.2)
c
c ---- Outputs to screen:
      write(*,'(a,"Latitude:",f6.2," Elevation:",f6.0)')xstanam,latt,
     &      elev
      write(*,3006,err=9999) p_ww
      write(*,3007,err=9999) p_wd
      write(*,3008,err=9999) t_max
      write(*,3009,err=9999) t_min
c ---- Outputs to screen:
 3001 format('LATT=',f7.2,' LONG=',f7.2,' YEARS=',f4.0)
 3002 format('ELEVATION =',f6.0)
 3003 format('MEAN P ',12f6.2)
 3004 format('S DEV P',12f6.2)
 3005 format('SKEW P ',12f6.2)
 3006 format('P(W|W) ',12f6.2)
 3007 format('P(W|D) ',12f6.2)
 3008 format('TMAX AV',12f6.2)
 3009 format('TMIN AV',12f6.2)
 3010 format('SD TMAX',12f6.2)
      close(10)
c
      x = matmet(0)
c
      xlatt = latt
      xelev = elev
      do 100, i=1, 12
c-------(There *is* a reason for doing this this way....)
        xp_ww(i) = p_ww(i)
        xp_wd(i) = p_wd(i)
        xmonpcp(i) = monpcp(i)
        xpcpamt(i) = pcpamt(i)
        xt_max(i) = t_max(i)
        xt_min(i) = t_min(i)
  100 continue
c
c-----initialize "keepers" array
      keepers(1) = xstanam
      keepmet(1) = 100000.0
      do 200, i=2, 10
        keepers(i) = 'xxx'
        keepmet(i) = 100000.0
  200 continue
c
      infile = 'AllStations.PAR'
      open(unit=10, file=infile, status='old', err=9096)
c
  500 continue
        read(10,1000,end=9097) latts
        if(latts .eq. ' LATT=') then
          backspace 10
          backspace 10
          read(10,1001,err=500) stanam, latt, elev, monpcp
          read(10,1002,err=500) p_ww, p_wd, t_max, t_min
c ---- Reading from FS station files:
 1000 format(a6)
 1001 format(a46/6x,f7.2/12x,f6.0/8x,12(f6.2)//)
 1002 format(4(8x,12(f6.2)/))
c
c----------Both stations from same (N or S) hemisphere
          if(xlatt*latt .ge. 0.0) then
            x = matmet(1)
          else
            x = matmet(-1)
          endif
c
c -------- If this station tops the worst one we'e kept, Keep it instead.
          if(x .lt. keepmet(10)) then
            keepmet(10) = x
            keepers(10) = stanam
c------------rerank the stations
            do 600, i=9, 1, -1
              if(keepmet(i).gt.keepmet(i+1)) then
                kmsave = keepmet(i+1)
                keepmet(i+1) = keepmet(i)
                keepmet(i) = kmsave
                snsave = stanam
                keepers(i+1) = keepers(i)
                keepers(i) = snsave
              endif
 600        continue
c ---------- Save "Best" Station's Parameters for display later
            if(keepers(1).eq.stanam) then
              lattsav = latt
              elevsav = elev
C     write(*,*) stanam, keepmet(1)
              do 610, i=1, 12
c--------------For opposite (N & S) hemispheres:
                if(xlatt*latt .lt. 0.0) then
                  io = i+6
                  if(io.gt.12) io = io-12
                else
                  io = i
                endif
                pwwsav(i) = p_ww(io)
                pwdsav(i) = p_wd(io)
c               pcpsav(i) = pcpamt(io)
                tmxsav(i) = t_max(io)
                tmnsav(i) = t_min(io)
 610          continue
            endif
          endif
        endif
      goto 500
c
c------ERRORS:
 9095 continue
      write(*,*) ' File:',xinfile,' not found.'
      write(*,*) ' It needs to be in the same directory (folder) as the 
     &FindMatch program.'
      goto 9999
 9096 continue
      write(*,*) ' File:',infile,' not found.'
      write(*,*) ' It needs to be in the same directory (folder) as the 
     &FindMatch program.'
      goto 9999
c
c------Display Matching Stations on the Screen.
 9097 continue
      write(*,*)
      write(*,*) 'Keepers:                                 Coefficient:'
      do 900 i=1, 10
        write(*,*) keepers(i), keepmet(i)
        if(i.eq.1) then
          write(*,'(46x,"Latitude:",f6.2," Elevation:",f6.0)')lattsav,
     &           elevsav
          write(*,3006) pwwsav
          write(*,3007) pwdsav
          write(*,3008) tmxsav
          write(*,3009) tmnsav
        endif
 900  continue
      write(outfile,'(a20)') xinfile
      i = 21
   10 continue
       i = i-1
      if(i.gt.1 .and. outfile(i:i) .ne.'.') goto 10
      outfile(i+1:i+3)='par'
c
      write(*,*) 'Press RETURN/ENTER to continue....'
      read(*,'(a)') respon
      write(*,*)
      write(*,*) 'Congratulations.  You have completed a run of the'
      write(*,*) 'FindMatch program.'
      write(*,*)
      write(*,*) 'The program will now build a file called ',outfile
      write(*,*) 'using the station that best matched yours: ',xinfile
      write(*,*)
      write(*,*) 'If you prefer not to use that file, you will need to'
      write(*,*) 'select a surrogate station file from those suggested,'
      write(*,*) '& replace its top 12 lines with the contents of file'
      write(*,'(1x,a)') xinfile
      write(*,*) 'If you do that, you will need to provide the correct'
      write(*,*) 'values for TYPE, TP5, and TP6, which default to zero'
      write(*,*) 'in the .top file.'
c
      x=wrtfile(outfile)
 9987 continue
      write(*,*) ' EXITING -- Error reading Station Name.'
      goto  9999
 9988 continue
      write(*,*) ' EXITING -- Error reading Latitude.'
      goto  9999
 9989 continue
      write(*,*) ' EXITING -- Error reading Elevation.'
      goto  9999
 9990 continue
      write(*,*) ' Elevation:',elev
      write(*,*) ' EXITING -- Error reading Mean of Pcp.'
      goto  9999
 9991 continue
      write(*,*) ' EXITING -- Error reading SD of Pcp.'
      goto  9999
 9992 continue
      write(*,*) ' EXITING -- Error reading Skew of Pcp.'
      goto  9999
 9993 continue
      write(*,*) ' EXITING -- Error reading P(W|W).'
      goto  9999
 9994 continue
      write(*,*) ' EXITING -- Error reading P(W|D).'
      goto  9999
 9995 continue
      write(*,*) ' EXITING -- Error reading Mean of Tmax.'
      goto  9999
 9996 continue
      write(*,*) ' EXITING -- Error reading Mean of Tmin.'
      goto  9999
 9997 continue
      write(*,*) ' EXITING -- Error reading SD of Tmax.'
      goto  9999
 9998 continue
      write(*,*) ' EXITING -- Error reading SD of Tmin.'
 9999 continue
      close(10)
      end
c
c
      real function matmet(flag)
c
c      Purpose: To calculate the statistic "matmet" which measures how
c               well the current station matches the target parameters.
c
c      Author: Charles R. Meyer
c
c      Date: 8/26/2003 - 8/26/2003; 9/11/2003
c
c
      integer flag
c
c
c        flag -- zero first time matmet is called; one if stations are in
c                same (N or S) hemisphere; minus one otherwise.
c
      include 'cbenchmk.inc'
      include 'cmatstat.inc'
c      matmet -- Metric used to determine quality of match
c      stanam -- Station Name
c        latt -- Station Latitude
c      monpcp -- Monthly average precip per event
c        p_ww -- Monthly probability of wet after wet
c        p_wd -- Monthly probability of wet after dry
c          pw -- Monthly probability of precip
c      pcpamt -- Average amount of precip per month
c       t_max -- Monthly average maximum temperature
c       t_min -- Monthly average minimum temperature
c      annpcp -- Average amount of precip per year
c      anntmx -- Average maximum temperature per year
c      anntmn -- Average minimum temperature per year
c
      matmet = 0.0
c     annpcp = 0.0
      if(xlatt .eq. 0.0) xlatt = 0.01
      if(xelev .eq. 0.0) xelev = 0.01
      do 100, i=1, 12
        if(xp_wd(i) .eq. 0.0) xp_wd(i) = 0.01
        if(xp_ww(i) .eq. 0.0) xp_ww(i) = 0.01
        if(flag .gt. 0) then
          matmet = matmet + 0.49*((p_wd(i) - xp_wd(i))**2)/xp_wd(i)
     &                    + 0.49*((p_ww(i) - xp_ww(i))**2)/xp_ww(i)
     &                    + 0.008333*((latt - xlatt)**2)/abs(xlatt)
     &                    + 0.008333*((elev - xelev)**2)/abs(xelev)
c
c--------Stations in opposite (N or S) hemispheres:
        else if(flag .lt. 0) then
C     write(*,*) 'OPPOSITE hemispheres -- MATMET'
          io = i+6
          if(io.gt.12) io = io-12
          matmet = matmet + 0.49*((p_wd(io) - xp_wd(i))**2)/xp_wd(i)
     &                    + 0.49*((p_ww(io) - xp_ww(i))**2)/xp_ww(i)
c----------------------------use offset from equator:
     &                    + 0.008333*((latt + xlatt)**2)/abs(xlatt)
     &                    + 0.008333*((elev - xelev)**2)/abs(xelev)
        else
c--------Target station to match existing station to:
          matmet = 0.0
        endif
  100 continue
c
      return
      end
c
      real function wrtfile(outfile)
c
c      Purpose: To write a Cligen station parameter file, using the derived
c               parameters in the .top file created by GenStPar, and the
c               best matching station found by FindMatch.  
c
c      Author: Charles R. Meyer
c
c      Date: 9/12/2003
c
c
      include 'cbenchmk.inc'
      include 'cmatstat.inc'
c
      character*20 outfile
      character*80 string
      character*6 latts
      character*8 astuff
      real stuff(12)
c      
      open(unit=11, file=xinfile, status='old', err=9095)
      open(unit=12, file=infile, status='old', err=9096)
c
      open(unit=13, file=outfile, status='unknown', err=9097)
c
      goto 200
c------ERRORS:
 9095 continue
      write(*,*) ' Unit 11 -- File:',xinfile,' not found.'
      write(*,*) ' It needs to be in the same directory (folder) as the 
     &FindMatch program.'
      goto 9999
 9096 continue
      write(*,*) ' Unit 12 -- File:',infile,' not found.'
      write(*,*) ' It needs to be in the same directory (folder) as the 
     &FindMatch program.'
      goto 9999
 9097 continue
      write(*,*) ' Unit 13 -- File:',outfile,' cannot be opened.'
      write(*,*) ' It needs to be in the same directory (folder) as the 
     &FindMatch program.'
      goto 9999
c
c------position to surrogate station in aggregate stations file.
  200 continue
        read(12,3000,end=9999) latts
 3000 format(a6)
        if(latts .eq. ' LATT=') then
          backspace 12
          backspace 12
          read(12,2000,end=9999,err=9999) stanam
C
C -- XXX -- Is this _right_? -- CRM -- 10/28/03:
 2000 format(a46)
c2000 format(a24)
          if(stanam.eq.keepers(1)) goto 9998
        endif
        read(12,*)
      goto 200
c
 9998 continue
      backspace 12
c     backspace 12
      do 100, i=1, 12
        string=''
        read(11,1000) string
c       write(*,1000) string
c       read(12,*)
 1000 format(a80)
c------write what's in the .top file
        if(i.ne.2 .and. i.ne.3) then
          read(12,*)
          write(13,1000) string
c         write(*,1000) string
c------use TYPE from surrogate station
        else if(i.eq.2) then
          read(string,1004) xlatt, long, years
 1004 format(' LATT=',f7.2,' LONG=',f7.2,' YEARS=',f4.0,' TYPE=',i2)
C         read(12,1005) itype
C1005 format(37x,' TYPE=', i2)
          read(12,1005) latt, itype
 1005 format(' LATT=',f7.2,24x,' TYPE=', i2)
          write(13,1004) xlatt, long, years, itype
c         write(*,1004) xlatt, long, years, itype
c------use elevation from actual data; TP5 & TP6 from surrogate station
        else if(i.eq.3) then
          read(string,1008) elev
          read(12,1007) tp5, tp6
 1007 format(18x,' TP5 =',f5.2,' TP6=',f5.2)
          write(13,1008) elev, tp5, tp6
 1008 format(' ELEVATION =',f6.0,' TP5 =',f5.2,' TP6=',f5.2)
        endif
  100 continue
c
c ----- If surrogate station is from opposite (N & S) hemisphere, do 6-mo shift.
C     write(*,*) xlatt, latt
      if(xlatt*latt .lt. 0.0) then
        do 250, line=13, 82
c----------Don't rearrange Tpeak:
          if(line .eq. 17) then
            read(12,'(a8,12f6.2)') astuff, stuff
            write(13,'(a8,12f6.2)') astuff, stuff
          else
            read(12,'(a8,12f6.2)') astuff, stuff
            write(13,'(a8,12f6.2)') astuff, (stuff(i), i=7,12), 
     &                             (stuff(i),i=1,6)
          endif
  250   continue
C     write(*,*) ' Flipping numbers.'
      endif
c------now copy the rest from the surrogate station
  300 continue
        string=''
        read(12,1000,end=9999) string
        write(13,1000) string
c       write(*,1000) string
        read(string,3000) latts
      if(latts .ne. ' LATT=') goto 300
c
c-----Write surrogate station name at bottom of the file.
      backspace 12
      write(13,'(" ***SURROGATE STATION:",a,"***")') keepers(1)
c     write(*,'("SURROGATE STATION:",a)') keepers(1)
c
 9999 continue
      close(11)
      close(12)
      close(13)
      return
      end
